==> contentsquare-2.1.0.tar.gz: ios/Classes/SwiftContentsquarePlugin.swift <==
import Flutter
import UIKit
import ContentsquareModule


// This is the protocol that the listeners passed by external bridges have to implement.
@objc
protocol ExternalBridgeInterface {
    @objc
    func takeSnapshot(parameters: [String: Any])
}

private class ExternalBridgeImpl: ExternalBridgeInterface {
    private var _handler: (_ rootPath: String, _ completion: @escaping (String?) -> Void) -> Void

    init(handler: @escaping (_ rootPath: String, _ completion: @escaping (String?) -> Void) -> Void) {
        _handler = handler
    }
   
  @objc
  func takeSnapshot(parameters: [String: Any]) {
    if let rootPath = parameters["rootPath"] as? String,
      let completion = parameters["completion"] as? (String?) -> Void {
        _handler(rootPath, completion)
    }
  }
}

// Expected elements takeSnapshot's parameters dictionary:
enum TakeSnapshotParam: String {
    
    // path of the rootView of the bridge on the native side. To be used to create TargetViewPaths when generating the snapshot
    case rootPath // as String - mandatory
    
    // completion block to be used in order to return the json of the generated snapshot asynchronoulsy
    case completion // as (json:String?) -> Void - mandatory
}

func registerExternalBridge(channel: FlutterMethodChannel) {
    let flutterTag = "|flutter|"
    Contentsquare.perform(
        NSSelectorFromString("_registerExternalBridgeWithParameters:"),
        with: [
          "interface": ExternalBridgeImpl(handler: { (rootPath: String, completion: @escaping (String?) -> Void) -> Void in
              channel.invokeMethod("requestFlutterSnapshot", arguments: ["rootPath": rootPath + flutterTag]) { result in
                  let rootViewBitmap = rootViewBitmap() ?? ""
                  var json = result as? String
                  json = json?.replacingOccurrences(of: "com_contentsquare_rootViewBitmapPlaceholder", with: rootViewBitmap)
                  completion(json)
              }
          }),
          "type": 1
        ]
      )
}

private func rootViewBitmap() -> String? {
  let flutterViews = UIApplication.shared.windows.flatMap {
    $0.allSubviews(where: { $0.className == "FlutterView" })
  }
  guard let flutterView = flutterViews.first else {
    return nil
  }
  return UIGraphicsImageRenderer(bounds: flutterView.bounds).image { _ in
    flutterView.drawHierarchy(in: flutterView.bounds, afterScreenUpdates: true)
  }.pngData()?.base64EncodedString()
}

public class SwiftContentsquarePlugin: NSObject, FlutterPlugin {
  private var methodCallHandlerImpl = MethodCallHandlerImpl()
  static private var _channel: FlutterMethodChannel!

  public static func register(with registrar: FlutterPluginRegistrar) {
    if (SwiftContentsquarePlugin._channel != nil) {
        NSLog(
            """
            CSLIB ⚠️ WARNING: SwiftContentsquarePlugin was registered twice. Please refer to the Contentsquare documentation's troubleshooting section for more information about what to do: https://docs.contentsquare.com/flutter/#troubleshooting
            """
        );
        return;
    }
    let channel = FlutterMethodChannel(name: "contentsquare", binaryMessenger: registrar.messenger())
    SwiftContentsquarePlugin._channel = channel
      
    let instance = SwiftContentsquarePlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
    registerExternalBridge(channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    switch call.method {
    case "send":
      methodCallHandlerImpl.send(call, result)
      break
    case "sendTransaction":
      methodCallHandlerImpl.sendTransaction(call, result)
      break
    case "sendDynamicVar":
      methodCallHandlerImpl.sendDynamicVar(call, result)
      break
    case "optIn":
      methodCallHandlerImpl.optIn(call, result)
      break
    case "optOut":
      methodCallHandlerImpl.optOut(call, result)
      break
    case "forgetMe":
      methodCallHandlerImpl.forgetMe(call, result)
      break
    case "getUserId":
      methodCallHandlerImpl.getUserId(call, result)
      break
    case "stopTracking":
      methodCallHandlerImpl.stopTracking(call, result)
      break
    case "resumeTracking":
      methodCallHandlerImpl.resumeTracking(call, result)
      break
    case "sendTouchEvent":
      methodCallHandlerImpl.sendTouchEvent(call, result)
      break
    default:
      result(FlutterMethodNotImplemented)
    }
  }
}


class MethodCallHandlerImpl {

  public func send(_ call: FlutterMethodCall, _ result: FlutterResult) {
    let arguments: [String : Any] = call.arguments as? [String : Any] ?? [:]
    guard let screenName = arguments["screenName"] as? String else {
      result(FlutterError(
        code: "MISSING_ARGUMENT",
        message: "Missing string for the screenName parameter",
        details: nil)
      )
      return
    }

    Contentsquare.send(screenViewWithName: screenName)
    result(nil)
  }

  public func sendTransaction(_ call: FlutterMethodCall, _ result: FlutterResult) {
    let arguments: [String : Any] = call.arguments as? [String : Any] ?? [:]

    let transactionId = arguments["transactionId"] as? String ?? nil
    let priceNumber = arguments["price"] as? NSNumber ?? nil
    let price = priceNumber?.floatValue
    let currency = arguments["currency"] as? String ?? nil

    if (price == nil) {
      result(FlutterError(
        code: "MISSING_ARGUMENT",
        message: "Missing double for the price parameter",
        details: nil)
      )
      return
    }
    if (currency == nil) {
      result(FlutterError(
        code: "MISSING_ARGUMENT",
        message: "Missing string for the currency parameter",
        details: nil)
      )
    }

    Contentsquare.send(transaction: CustomerTransaction(id: transactionId, value: price!, currency: currency!))
    result(nil)
  }

  public func sendDynamicVar(_ call: FlutterMethodCall, _ result: FlutterResult) {
    let arguments: [String : Any] = call.arguments as? [String : Any] ?? [:]

    let key = arguments["key"] as? String ?? nil
    let stringValue = arguments["stringValue"] as? String ?? nil
    let intValue = arguments["intValue"] as? UInt32 ?? nil

    if (key == nil) {
      result(FlutterError(
        code: "MISSING_ARGUMENT",
        message: "Missing string for the key parameter",
        details: nil)
      )
      return
    }
    do {
      if (stringValue != nil) {
        try Contentsquare.send(dynamicVar: DynamicVar(key: key!, value: stringValue!))
        result(nil)
        return
      }
      if (intValue != nil) {
        try Contentsquare.send(dynamicVar: DynamicVar(key: key!, value: intValue!))
        result(nil)
        return
      }
    } catch {
      result(FlutterError(
        code: "DYNAMIC_VAR_ERROR",
        message: "Error while sending the dynamic variable. Please make sure that you have defined the variable in the Contentsquare console",
        details: nil)
      )
      return
    }
    result(FlutterError(
        code: "MISSING_ARGUMENT",
        message: "Missing stringValue or intValue parameter",
        details: nil)
      )
  }

  public func optIn(_ call: FlutterMethodCall, _ result: FlutterResult) {
    Contentsquare.optIn()
    result(nil)
  }

  public func optOut(_ call: FlutterMethodCall, _ result: FlutterResult) {
    Contentsquare.optOut()
    result(nil)
  }

  public func forgetMe(_ call: FlutterMethodCall, _ result: FlutterResult) {
    Contentsquare.forgetMe()
    result(nil)
  }

  public func getUserId(_ call: FlutterMethodCall, _ result: FlutterResult) {
    result(Contentsquare.userID)
  }

  public func stopTracking(_ call: FlutterMethodCall, _ result: FlutterResult) {
    Contentsquare.stopTracking()
    result(nil)
  }

  public func resumeTracking(_ call: FlutterMethodCall, _ result: FlutterResult) {
    Contentsquare.resumeTracking()
    result(nil)
  }

  public func sendTouchEvent(_ call: FlutterMethodCall, _ result: FlutterResult) {
    let arguments: [String : Any] = call.arguments as? [String : Any] ?? [:]

    let body = arguments["body"] as? String
    
    if (body == nil) {
      result(FlutterError(
        code: "MISSING_ARGUMENT",
        message: "Missing string for the body parameter",
        details: nil)
      )
      return
    }

    Contentsquare.perform(NSSelectorFromString("_sendEventWithJson:"), with: body)

    result(nil)
  }
}

extension UIView {

  func allSubviews(where predicate: (UIView) -> Bool) -> [UIView] {
    let views: [UIView] = predicate(self) ? [self] : []
    return views + subviews.flatMap { $0.allSubviews(where: predicate) }
  }

  var className: String {
    String(describing: type(of: self))
  }
}

==> contentsquare-2.3.0.tar.gz: ios/Classes/SwiftContentsquarePlugin.swift <==
import Flutter
import UIKit
import ContentsquareModule


// This is the protocol that the listeners passed by external bridges have to implement.
@objc
protocol ExternalBridgeInterface {
    @objc
    func takeSnapshot(parameters: [String: Any])
}

private class ExternalBridgeImpl: ExternalBridgeInterface {
    private var _handler: (_ rootPath: String, _ completion: @escaping (String?) -> Void) -> Void

    init(handler: @escaping (_ rootPath: String, _ completion: @escaping (String?) -> Void) -> Void) {
        _handler = handler
    }
   
  @objc
  func takeSnapshot(parameters: [String: Any]) {
    if let rootPath = parameters["rootPath"] as? String,
      let completion = parameters["completion"] as? (String?) -> Void {
        _handler(rootPath, completion)
    }
  }
}

// Expected elements takeSnapshot's parameters dictionary:
enum TakeSnapshotParam: String {
    
    // path of the rootView of the bridge on the native side. To be used to create TargetViewPaths when generating the snapshot
    case rootPath // as String - mandatory
    
    // completion block to be used in order to return the json of the generated snapshot asynchronoulsy
    case completion // as (json:String?) -> Void - mandatory
}

func registerExternalBridge(channel: FlutterMethodChannel) {
    let flutterTag = "|flutter|"
    Contentsquare.perform(
        NSSelectorFromString("_registerExternalBridgeWithParameters:"),
        with: [
          "interface": ExternalBridgeImpl(handler: { (rootPath: String, completion: @escaping (String?) -> Void) -> Void in
              channel.invokeMethod("requestFlutterSnapshot", arguments: ["rootPath": rootPath + flutterTag]) { result in
                  let rootViewBitmap = rootViewBitmap() ?? ""
                  var json = result as? String
                  json = json?.replacingOccurrences(of: "com_contentsquare_rootViewBitmapPlaceholder", with: rootViewBitmap)
                  completion(json)
              }
          }),
          "type": 1
        ]
      )
}

private func rootViewBitmap() -> String? {
  let flutterViews = UIApplication.shared.windows.flatMap {
    $0.allSubviews(where: { $0.className == "FlutterView" })
  }
  guard let flutterView = flutterViews.first else {
    return nil
  }
  return UIGraphicsImageRenderer(bounds: flutterView.bounds).image { _ in
    flutterView.drawHierarchy(in: flutterView.bounds, afterScreenUpdates: true)
  }.pngData()?.base64EncodedString()
}

public class SwiftContentsquarePlugin: NSObject, FlutterPlugin {
  private var methodCallHandlerImpl = MethodCallHandlerImpl()
  static private var _channel: FlutterMethodChannel!

  public static func register(with registrar: FlutterPluginRegistrar) {
    if (SwiftContentsquarePlugin._channel != nil) {
        NSLog(
            """
            CSLIB ⚠️ WARNING: SwiftContentsquarePlugin was registered twice. Please refer to the Contentsquare documentation's troubleshooting section for more information about what to do: https://docs.contentsquare.com/flutter/#troubleshooting
            """
        );
        return;
    }
    let channel = FlutterMethodChannel(name: "contentsquare", binaryMessenger: registrar.messenger())
    SwiftContentsquarePlugin._channel = channel
      
    let instance = SwiftContentsquarePlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
    registerExternalBridge(channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    switch call.method {
    case "send":
      methodCallHandlerImpl.send(call, result)
      break
    case "sendTransaction":
      methodCallHandlerImpl.sendTransaction(call, result)
      break
    case "sendDynamicVar":
      methodCallHandlerImpl.sendDynamicVar(call, result)
      break
    case "optIn":
      methodCallHandlerImpl.optIn(call, result)
      break
    case "optOut":
      methodCallHandlerImpl.optOut(call, result)
      break
    case "forgetMe":
      methodCallHandlerImpl.forgetMe(call, result)
      break
    case "getUserId":
      methodCallHandlerImpl.getUserId(call, result)
      break
    case "stopTracking":
      methodCallHandlerImpl.stopTracking(call, result)
      break
    case "resumeTracking":
      methodCallHandlerImpl.resumeTracking(call, result)
      break
    case "sendTouchEvent":
      methodCallHandlerImpl.sendTouchEvent(call, result)
      break
    default:
      result(FlutterMethodNotImplemented)
    }
  }
}


class MethodCallHandlerImpl {

  public func send(_ call: FlutterMethodCall, _ result: FlutterResult) {
    let arguments: [String : Any] = call.arguments as? [String : Any] ?? [:]
    guard let screenName = arguments["screenName"] as? String else {
      result(FlutterError(
        code: "MISSING_ARGUMENT",
        message: "Missing string for the screenName parameter",
        details: nil)
      )
      return
    }

    Contentsquare.send(screenViewWithName: screenName)
    result(nil)
  }

  public func sendTransaction(_ call: FlutterMethodCall, _ result: FlutterResult) {
    let arguments: [String : Any] = call.arguments as? [String : Any] ?? [:]

    let transactionId = arguments["transactionId"] as? String ?? nil
    let priceNumber = arguments["price"] as? NSNumber ?? nil
    let price = priceNumber?.floatValue
    let currency = arguments["currency"] as? String ?? nil

    if (price == nil) {
      result(FlutterError(
        code: "MISSING_ARGUMENT",
        message: "Missing double for the price parameter",
        details: nil)
      )
      return
    }
    if (currency == nil) {
      result(FlutterError(
        code: "MISSING_ARGUMENT",
        message: "Missing string for the currency parameter",
        details: nil)
      )
    }

    Contentsquare.send(transaction: CustomerTransaction(id: transactionId, value: price!, currency: currency!))
    result(nil)
  }

  public func sendDynamicVar(_ call: FlutterMethodCall, _ result: FlutterResult) {
    let arguments: [String : Any] = call.arguments as? [String : Any] ?? [:]

    let key = arguments["key"] as? String ?? nil
    let stringValue = arguments["stringValue"] as? String ?? nil
    let intValue = arguments["intValue"] as? NSNumber ?? nil

    if (key == nil) {
      result(FlutterError(
        code: "MISSING_ARGUMENT",
        message: "Missing string for the key parameter",
        details: nil)
      )
      return
    }
    do {
      if (stringValue != nil) {
        try Contentsquare.send(dynamicVar: DynamicVar(key: key!, value: stringValue!))
        result(nil)
        return
      }
      if (intValue != nil) {
        try Contentsquare.send(dynamicVar: DynamicVar(key: key!, value: intValue!.uint32Value))
        result(nil)
        return
      }
    } catch {
      result(FlutterError(
        code: "DYNAMIC_VAR_ERROR",
        message: "Error while sending the dynamic variable. Please make sure that you have defined the variable in the Contentsquare console",
        details: nil)
      )
      return
    }
    result(FlutterError(
        code: "MISSING_ARGUMENT",
        message: "Missing stringValue or intValue parameter",
        details: nil)
      )
  }

  public func optIn(_ call: FlutterMethodCall, _ result: FlutterResult) {
    Contentsquare.optIn()
    result(nil)
  }

  public func optOut(_ call: FlutterMethodCall, _ result: FlutterResult) {
    Contentsquare.optOut()
    result(nil)
  }

  public func forgetMe(_ call: FlutterMethodCall, _ result: FlutterResult) {
    Contentsquare.forgetMe()
    result(nil)
  }

  public func getUserId(_ call: FlutterMethodCall, _ result: FlutterResult) {
    result(Contentsquare.userID)
  }

  public func stopTracking(_ call: FlutterMethodCall, _ result: FlutterResult) {
    Contentsquare.stopTracking()
    result(nil)
  }

  public func resumeTracking(_ call: FlutterMethodCall, _ result: FlutterResult) {
    Contentsquare.resumeTracking()
    result(nil)
  }

  public func sendTouchEvent(_ call: FlutterMethodCall, _ result: FlutterResult) {
    let arguments: [String : Any] = call.arguments as? [String : Any] ?? [:]

    let body = arguments["body"] as? String
    
    if (body == nil) {
      result(FlutterError(
        code: "MISSING_ARGUMENT",
        message: "Missing string for the body parameter",
        details: nil)
      )
      return
    }

    Contentsquare.perform(NSSelectorFromString("_sendEventWithJson:"), with: body)

    result(nil)
  }
}

extension UIView {

  func allSubviews(where predicate: (UIView) -> Bool) -> [UIView] {
    let views: [UIView] = predicate(self) ? [self] : []
    return views + subviews.flatMap { $0.allSubviews(where: predicate) }
  }

  var className: String {
    String(describing: type(of: self))
  }
}
